//
// derived from AVR TM1638 "Library" v1.01
//
#include <avr/io.h>
#include <util/delay.h>
#include "HC595.h"
#include "HC595Font.h"

#define TM_SCK_LOW()            (TM_OUT &= ~TM_BIT_SCK)
#define TM_SCK_HIGH()           (TM_OUT |= TM_BIT_SCK)
#define TM_RCK_LOW()            (TM_OUT &= ~TM_BIT_RCK)
#define TM_RCK_HIGH()           (TM_OUT |= TM_BIT_RCK)
#define TM_DIO_LOW()            (TM_OUT &= ~TM_BIT_DIO)
#define TM_DIO_HIGH()           (TM_OUT |= TM_BIT_DIO)

namespace
{
    void port_setup()
    {
        TM_DDR |= TM_BIT_RCK | TM_BIT_SCK | TM_BIT_DIO;
        TM_OUT |= TM_BIT_RCK | TM_BIT_SCK;
    }

    void send(uint8_t b)
    {
        for (uint8_t i = 8; i; --i, b >>= 1)
        {
            TM_SCK_LOW();
            if (b & 1)
                TM_DIO_HIGH();
            else
                TM_DIO_LOW();
            TM_SCK_HIGH();
        }
    }

    void send_cmd(const uint8_t cmd)
    {
        TM_RCK_LOW();
        send(cmd);
        TM_RCK_HIGH();
    }

    void send_data(const uint8_t addr, const uint8_t data)
    {
        TM_RCK_LOW();
        send(addr);
        send(data);
        TM_RCK_HIGH();
    }

    uint8_t offset_digits(uint16_t num)
    {
        uint8_t digits = 0;
        while (num >= 10)
        {
            num /= 10;
            ++digits;
        }
        return digits;
    }
}

HC595::HC595()
    : _dotMask(0)
{
    port_setup();

    clear();
}

void HC595::clear()
{
        send_data(0xFF, TM_BLANK);
}

void HC595::setByte(const uint8_t pos, const uint8_t value)
{
	int Num_Pos_B = 0;
	switch (pos)
    {
                case 7:Num_Pos_B = 1;break;
                case 6:Num_Pos_B = 2;break;
                case 5:Num_Pos_B = 4;break;
                case 4:Num_Pos_B = 8;break;
                case 3:Num_Pos_B = 16;break;
                case 2:Num_Pos_B = 32;break;
                case 1:Num_Pos_B = 64;break;
                case 0:Num_Pos_B = 128;break;
                default: Num_Pos_B = 0;break;
    };
    send_data(Num_Pos_B, value & (_dotMask & Num_Pos_B ? TM_DOT : 0xFF ));
}

void HC595::setDigit(const uint8_t pos, const uint8_t value)
{
    setByte(pos, pgm_read_byte(&TM_DIGITS[value & 0xF]));
}

void HC595::setNumber(uint32_t number, uint8_t offset, const uint8_t align)
{
    if (align == TM_LEFT)
        offset += offset_digits(number);

    while (number && offset != 0xFF)
    {
        setDigit(offset--, number % 10);
        number /= 10;
    }
}

void HC595::setNumberPad(uint32_t number, uint8_t offset, uint8_t width, const uint8_t pad)
{
    while (number && width-- && offset != 0xFF)
    {
        setDigit(offset--, number % 10);
        number /= 10;
    }

    while (width -- && offset != 0xFF)
        setByte(offset--, pad);
}

void HC595::setNumberHex(uint32_t number, uint8_t offset, uint8_t width, const uint8_t pad)
{
    while (number && width-- && offset != 0xFF)
    {
        setDigit(offset--, number & 0x0F);
        number >>= 4;
    }

    while (width -- && offset != 0xFF)
        setByte(offset--, pad);
}

void HC595::setChar(const uint8_t pos, const char value)
{
    const uint8_t b = HC595_map_char(value);
    if (b)
        setByte(pos, b);

    else if (value >= 'a' && value <= 'z')
        setByte(pos, pgm_read_byte(&TM_DIGITS[value - 'a' + 10]));

    else if (value >= 'A' && value <= 'Z')
        setByte(pos, pgm_read_byte(&TM_DIGITS[value - 'A' + 10]));

    else if (value >= '0' && value <= '9')
        setByte(pos, pgm_read_byte(&TM_DIGITS[value - '0']));
}

void HC595::setChars(const char* value, uint8_t offset)
{
    while (*value)
        setChar(offset++, *value++);
}

void HC595::setDots(const uint8_t mask)
{
    _dotMask = mask;
}
